@@ -3,10 +3,10 @@
 /* The name of this package (not localized) */
 const PACKAGE_NAME = 'gnome-shell';
 /* The version of this package */
-const PACKAGE_VERSION = '3.3.2';
+const PACKAGE_VERSION = '3.3.3';
 /* The version of GJS we're linking to */
-const GJS_VERSION = '1.31.0';
+const GJS_VERSION = '1.31.6';
 /* 1 if gnome-bluetooth is available, 0 otherwise */
 const HAVE_BLUETOOTH = 1;
 /* The system TLS CA list */
-const SHELL_SYSTEM_CA_FILE = '/etc/ssl/certs/ca-certificates.crt';
+const SHELL_SYSTEM_CA_FILE = '/etc/pki/tls/certs/ca-bundle.crt';
