@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
@@ -9,11 +9,13 @@ const Clutter = imports.gi.Clutter;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
 
+const Params = imports.misc.params;
 const Util = imports.misc.util;
 const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 const Calendar = imports.ui.calendar;
+const UPowerGlib = imports.gi.UPowerGlib;
 
 // in org.gnome.desktop.interface
 const CLOCK_FORMAT_KEY        = 'clock-format';
@@ -38,29 +40,26 @@ function _onVertSepRepaint (area)
     cr.stroke();
 };
 
-function DateMenuButton() {
-    this._init();
-}
+const DateMenuButton = new Lang.Class({
+    Name: 'DateMenuButton',
+    Extends: PanelMenu.Button,
 
-DateMenuButton.prototype = {
-    __proto__: PanelMenu.Button.prototype,
+    _init: function(params) {
+        params = Params.parse(params, { showEvents: true });
 
-    _init: function() {
         let item;
         let hbox;
         let vbox;
 
-        this._eventSource = new Calendar.DBusEventSource();
-
         let menuAlignment = 0.25;
         if (St.Widget.get_default_direction() == St.TextDirection.RTL)
             menuAlignment = 1.0 - menuAlignment;
-        PanelMenu.Button.prototype._init.call(this, menuAlignment);
+        this.parent(menuAlignment);
 
         this._clock = new St.Label();
-        this.actor.set_child(this._clock);
+        this.actor.add_actor(this._clock);
 
-        hbox = new St.BoxLayout({name: 'calendarArea'});
+        hbox = new St.BoxLayout({name: 'calendarArea' });
         this.menu.addActor(hbox);
 
         // Fill up the first column
@@ -73,43 +72,58 @@ DateMenuButton.prototype = {
         this._date.style_class = 'datemenu-date-label';
         vbox.add(this._date);
 
-        this._eventList = new Calendar.EventsList(this._eventSource);
+        if (params.showEvents) {
+            this._eventSource = new Calendar.DBusEventSource();
+            this._eventList = new Calendar.EventsList(this._eventSource);
+        } else {
+            this._eventSource = null;
+            this._eventList = null;
+        }
 
         // Calendar
         this._calendar = new Calendar.Calendar(this._eventSource);
+
         this._calendar.connect('selected-date-changed',
                                Lang.bind(this, function(calendar, date) {
+                                  // we know this._eventList is defined here, because selected-data-changed
+                                  // only gets emitted when the user clicks a date in the calendar,
+                                  // and the calender makes those dates unclickable when instantiated with
+                                  // a null event source
                                    this._eventList.setDate(date);
                                }));
         vbox.add(this._calendar.actor);
 
-        item = new PopupMenu.PopupSeparatorMenuItem();
-        item.setColumnWidths(1);
-        vbox.add(item.actor, {y_align: St.Align.END, expand: true, y_fill: false});
-        item = new PopupMenu.PopupMenuItem(_("Date and Time Settings"));
-        item.connect('activate', Lang.bind(this, this._onPreferencesActivate));
-        item.actor.can_focus = false;
-        vbox.add(item.actor);
+        item = this.menu.addSettingsAction(_("Date and Time Settings"), 'gnome-datetime-panel.desktop');
+        if (item) {
+            let separator = new PopupMenu.PopupSeparatorMenuItem();
+            separator.setColumnWidths(1);
+            vbox.add(separator.actor, {y_align: St.Align.END, expand: true, y_fill: false});
 
-        // Add vertical separator
+            item.actor.can_focus = false;
+            item.actor.reparent(vbox);
+        }
 
-        item = new St.DrawingArea({ style_class: 'calendar-vertical-separator',
-                                    pseudo_class: 'highlighted' });
-        item.connect('repaint', Lang.bind(this, _onVertSepRepaint));
-        hbox.add(item);
+        if (params.showEvents) {
+            // Add vertical separator
 
-        // Fill up the second column
+            item = new St.DrawingArea({ style_class: 'calendar-vertical-separator',
+                                        pseudo_class: 'highlighted' });
+            item.connect('repaint', Lang.bind(this, _onVertSepRepaint));
+            hbox.add(item);
 
-        vbox = new St.BoxLayout({vertical: true});
-        hbox.add(vbox, { expand: true });
+            // Fill up the second column
+            vbox = new St.BoxLayout({name:     'calendarEventsArea',
+                                     vertical: true});
+            hbox.add(vbox, { expand: true });
 
-        // Event list
-        vbox.add(this._eventList.actor, { expand: true });
+            // Event list
+            vbox.add(this._eventList.actor, { expand: true });
 
-        item = new PopupMenu.PopupMenuItem(_("Open Calendar"));
-        item.connect('activate', Lang.bind(this, this._onOpenCalendarActivate));
-        item.actor.can_focus = false;
-        vbox.add(item.actor, {y_align: St.Align.END, expand: true, y_fill: false});
+            item = new PopupMenu.PopupMenuItem(_("Open Calendar"));
+            item.connect('activate', Lang.bind(this, this._onOpenCalendarActivate));
+            item.actor.can_focus = false;
+            vbox.add(item.actor, {y_align: St.Align.END, expand: true, y_fill: false});
+        }
 
         // Whenever the menu is opened, select today
         this.menu.connect('open-state-changed', Lang.bind(this, function(menu, isOpen) {
@@ -142,6 +156,10 @@ DateMenuButton.prototype = {
         this._desktopSettings.connect('changed', Lang.bind(this, this._updateClockAndDate));
         this._clockSettings.connect('changed', Lang.bind(this, this._updateClockAndDate));
 
+        // https://bugzilla.gnome.org/show_bug.cgi?id=655129
+        this._upClient = new UPowerGlib.Client();
+        this._upClient.connect('notify-resume', Lang.bind(this, this._updateClockAndDate));
+
         // Start the clock
         this._updateClockAndDate();
     },
@@ -157,12 +175,12 @@ DateMenuButton.prototype = {
         switch (format) {
             case '24h':
                 if (showDate)
-	            /* Translators: This is the time format with date used
+                    /* Translators: This is the time format with date used
                        in 24-hour mode. */
                     clockFormat = showSeconds ? _("%a %b %e, %R:%S")
                                               : _("%a %b %e, %R");
                 else
-	            /* Translators: This is the time format without date used
+                    /* Translators: This is the time format without date used
                        in 24-hour mode. */
                     clockFormat = showSeconds ? _("%a %R:%S")
                                               : _("%a %R");
@@ -170,12 +188,12 @@ DateMenuButton.prototype = {
             case '12h':
             default:
                 if (showDate)
-	            /* Translators: This is a time format with date used
+                    /* Translators: This is a time format with date used
                        for AM/PM. */
                     clockFormat = showSeconds ? _("%a %b %e, %l:%M:%S %p")
                                               : _("%a %b %e, %l:%M %p");
                 else
-	            /* Translators: This is a time format without date used
+                    /* Translators: This is a time format without date used
                        for AM/PM. */
                     clockFormat = showSeconds ? _("%a %l:%M:%S %p")
                                               : _("%a %l:%M %p");
@@ -196,16 +214,26 @@ DateMenuButton.prototype = {
         return false;
     },
 
-    _onPreferencesActivate: function() {
-        this.menu.close();
-        Main.overview.hide();
-        let app = Shell.AppSystem.get_default().get_app('gnome-datetime-panel.desktop');
-        app.activate(-1);
-    },
-
     _onOpenCalendarActivate: function() {
         this.menu.close();
-        // TODO: pass the selected day
-        Util.spawn(['evolution', '-c', 'calendar']);
+        let calendarSettings = new Gio.Settings({ schema: 'org.gnome.desktop.default-applications.office.calendar' });
+        let tool = calendarSettings.get_string('exec');
+        if (tool.length == 0 || tool == 'evolution') {
+            // TODO: pass the selected day
+            Util.spawn(['evolution', '-c', 'calendar']);
+        } else {
+            let needTerm = calendarSettings.get_boolean('needs-term');
+            if (needTerm) {
+                let terminalSettings = new Gio.Settings({ schema: 'org.gnome.desktop.default-applications.terminal' });
+                let term = terminalSettings.get_string('exec');
+                let arg = terminalSettings.get_string('exec-arg');
+                if (arg != '')
+                    Util.spawn([term, arg, tool]);
+                else
+                    Util.spawn([term, tool]);
+            } else {
+                Util.spawnCommandLine(tool)
+            }
+        }
     }
-};
+});
