@@ -1,6 +1,5 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 const ByteArray = imports.byteArray;
-const DBus = imports.dbus;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Lang = imports.lang;
@@ -98,15 +97,12 @@ function ssidToLabel(ssid) {
     return label;
 }
 
-function NMNetworkMenuItem() {
-    this._init.apply(this, arguments);
-}
-
-NMNetworkMenuItem.prototype = {
-    __proto__: PopupMenu.PopupBaseMenuItem.prototype,
+const NMNetworkMenuItem = new Lang.Class({
+    Name: 'NMNetworkMenuItem',
+    Extends: PopupMenu.PopupBaseMenuItem,
 
     _init: function(accessPoints, title, params) {
-        PopupMenu.PopupBaseMenuItem.prototype._init.call(this, params);
+        this.parent(params);
 
         accessPoints = sortAccessPoints(accessPoints);
         this.bestAP = accessPoints[0];
@@ -185,21 +181,18 @@ NMNetworkMenuItem.prototype = {
             apObj.updateId = 0;
         }
 
-        PopupMenu.PopupBaseMenuItem.prototype.destroy.call(this);
+        this.parent();
     }
-};
+});
 
-function NMWiredSectionTitleMenuItem() {
-    this._init.apply(this, arguments);
-}
-
-NMWiredSectionTitleMenuItem.prototype = {
-    __proto__: PopupMenu.PopupSwitchMenuItem.prototype,
+const NMWiredSectionTitleMenuItem = new Lang.Class({
+    Name: 'NMWiredSectionTitleMenuItem',
+    Extends: PopupMenu.PopupSwitchMenuItem,
 
     _init: function(label, params) {
         params = params || { };
         params.style_class = 'popup-subtitle-menu-item';
-        PopupMenu.PopupSwitchMenuItem.prototype._init.call(this, label, false, params);
+        this.parent(label, false, params);
     },
 
     updateForDevice: function(device) {
@@ -212,7 +205,7 @@ NMWiredSectionTitleMenuItem.prototype = {
     },
 
     activate: function(event) {
-        PopupMenu.PopupSwitchMenuItem.prototype.activate.call(this, event);
+        this.parent(event);
 
         if (!this._device) {
             log('Section title activated when there is more than one device, should be non reactive');
@@ -231,19 +224,16 @@ NMWiredSectionTitleMenuItem.prototype = {
         else
             this._device.deactivate();
     }
-};
+});
 
-function NMWirelessSectionTitleMenuItem() {
-    this._init.apply(this, arguments);
-}
-
-NMWirelessSectionTitleMenuItem.prototype = {
-    __proto__: PopupMenu.PopupSwitchMenuItem.prototype,
+const NMWirelessSectionTitleMenuItem = new Lang.Class({
+    Name: 'NMWirelessSectionTitleMenuItem',
+    Extends: PopupMenu.PopupSwitchMenuItem,
 
     _init: function(client, property, title, params) {
         params = params || { };
         params.style_class = 'popup-subtitle-menu-item';
-        PopupMenu.PopupSwitchMenuItem.prototype._init.call(this, title, false, params);
+        this.parent(title, false, params);
 
         this._client = client;
         this._property = property + '_enabled';
@@ -259,9 +249,11 @@ NMWirelessSectionTitleMenuItem.prototype = {
     updateForDevice: function(device) {
         // we show the switch
         // - if there not just one device
-        // - if the switch is off
+        // - if the switch is off (but it can be turned on)
         // - if the device is activated or disconnected
-        if (device && this._softwareEnabled && this._hardwareEnabled) {
+        if (!this._hardwareEnabled) {
+            this.setStatus(_("hardware disabled"));
+        } else if (device && this._softwareEnabled) {
             let text = device.getStatusLabel();
             this.setStatus(text);
         } else
@@ -269,7 +261,7 @@ NMWirelessSectionTitleMenuItem.prototype = {
     },
 
     activate: function(event) {
-        PopupMenu.PopupSwitchMenuItem.prototype.activate.call(this, event);
+        this.parent(event);
 
         this._client[this._setEnabledFunc](this._switch.state);
     },
@@ -286,13 +278,12 @@ NMWirelessSectionTitleMenuItem.prototype = {
 
         this.emit('enabled-changed', enabled);
     }
-};
+});
 
-function NMDevice() {
-    throw new TypeError('Instantanting abstract class NMDevice');
-}
+const NMDevice = new Lang.Class({
+    Name: 'NMDevice',
+    Abstract: true,
 
-NMDevice.prototype = {
     _init: function(client, device, connections) {
         this.device = device;
         if (device) {
@@ -338,7 +329,7 @@ NMDevice.prototype = {
         }
         this.section = new PopupMenu.PopupMenuSection();
 
-        this._createSection();
+        this._deferredWorkId = Main.initializeDeferredWork(this.section.actor, Lang.bind(this, this._createSection));
     },
 
     destroy: function() {
@@ -405,16 +396,29 @@ NMDevice.prototype = {
         this._activeConnection = activeConnection;
 
         this._clearSection();
-        this._createSection();
+        this._queueCreateSection();
     },
 
     checkConnection: function(connection) {
-        let exists = this._findConnection(connection._uuid) != -1;
+        let pos = this._findConnection(connection._uuid);
+        let exists = pos != -1;
         let valid = this.connectionValid(connection);
+
         if (exists && !valid)
             this.removeConnection(connection);
         else if (!exists && valid)
             this.addConnection(connection);
+        else if (exists && valid) {
+            // propagate changes and update the UI
+
+            if (this._connections[pos].timestamp != connection._timestamp) {
+                this._connections[pos].timestamp = connection._timestamp;
+                this._connections.sort(this._connectionSortFunction);
+
+                this._clearSection();
+                this._queueCreateSection();
+            }
+        }
     },
 
     addConnection: function(connection) {
@@ -429,7 +433,7 @@ NMDevice.prototype = {
         this._connections.sort(this._connectionSortFunction);
 
         this._clearSection();
-        this._createSection();
+        this._queueCreateSection();
     },
 
     removeConnection: function(connection) {
@@ -453,7 +457,7 @@ NMDevice.prototype = {
             // (or in the case of NMDeviceWired, we want to hide
             // the only explicit connection)
             this._clearSection();
-            this._createSection();
+            this._queueCreateSection();
         }
     },
 
@@ -537,6 +541,11 @@ NMDevice.prototype = {
         return -1;
     },
 
+    _queueCreateSection: function() {
+        this._clearSection();
+        Main.queueDeferredWork(this._deferredWorkId);
+    },
+
     _clearSection: function() {
         // Clear everything
         this.section.removeAll();
@@ -631,7 +640,7 @@ NMDevice.prototype = {
         this._updateStatusItem();
 
         this._clearSection();
-        this._createSection();
+        this._queueCreateSection();
         this.emit('state-changed');
     },
 
@@ -674,26 +683,23 @@ NMDevice.prototype = {
 
         return out;
     }
-};
+});
 Signals.addSignalMethods(NMDevice.prototype);
 
 
-function NMDeviceWired() {
-    this._init.apply(this, arguments);
-}
-
-NMDeviceWired.prototype = {
-    __proto__: NMDevice.prototype,
+const NMDeviceWired = new Lang.Class({
+    Name: 'NMDeviceWired',
+    Extends: NMDevice,
 
     _init: function(client, device, connections) {
         this._autoConnectionName = _("Auto Ethernet");
         this.category = NMConnectionCategory.WIRED;
 
-        NMDevice.prototype._init.call(this, client, device, connections);
+        this.parent(client, device, connections);
     },
 
     _createSection: function() {
-        NMDevice.prototype._createSection.call(this);
+        this.parent();
 
         // if we have only one connection (normal or automatic)
         // we hide the connection list, and use the switch to control
@@ -718,14 +724,11 @@ NMDeviceWired.prototype = {
         }));
         return connection;
     }
-};
+});
 
-function NMDeviceModem() {
-    this._init.apply(this, arguments);
-}
-
-NMDeviceModem.prototype = {
-    __proto__: NMDevice.prototype,
+const NMDeviceModem = new Lang.Class({
+    Name: 'NMDeviceModem',
+    Extends: NMDevice,
 
     _init: function(client, device, connections) {
         let is_wwan = false;
@@ -774,7 +777,7 @@ NMDeviceModem.prototype = {
             }));
         }
 
-        NMDevice.prototype._init.call(this, client, device, connections);
+        this.parent(client, device, connections);
     },
 
     setEnabled: function(enabled) {
@@ -787,7 +790,7 @@ NMDeviceModem.prototype = {
                 this.statusItem.setStatus(this.getStatusLabel());
         }
 
-        NMDevice.prototype.setEnabled.call(this, enabled);
+        this.parent(enabled);
     },
 
     get connected() {
@@ -804,7 +807,7 @@ NMDeviceModem.prototype = {
             this._signalQualityId = 0;
         }
 
-        NMDevice.prototype.destroy.call(this);
+        this.parent();
     },
 
     _getSignalIcon: function() {
@@ -825,13 +828,13 @@ NMDeviceModem.prototype = {
             this.section.addMenuItem(this._operatorItem);
         }
 
-        NMDevice.prototype._createSection.call(this);
+        this.parent();
     },
 
     _clearSection: function() {
         this._operatorItem = null;
 
-        NMDevice.prototype._clearSection.call(this);
+        this.parent();
     },
 
     _createAutomaticConnection: function() {
@@ -841,14 +844,11 @@ NMDeviceModem.prototype = {
                     'connect-3g', this.device.get_path()]);
         return null;
     }
-};
+});
 
-function NMDeviceBluetooth() {
-    this._init.apply(this, arguments);
-}
-
-NMDeviceBluetooth.prototype = {
-    __proto__: NMDevice.prototype,
+const NMDeviceBluetooth = new Lang.Class({
+    Name: 'NMDeviceBluetooth',
+    Extends: NMDevice,
 
     _init: function(client, device, connections) {
         this._autoConnectionName = this._makeConnectionName(device);
@@ -856,7 +856,7 @@ NMDeviceBluetooth.prototype = {
 
         this.category = NMConnectionCategory.WWAN;
 
-        NMDevice.prototype._init.call(this, client, device, connections);
+        this.parent(client, device, connections);
     },
 
     _createAutomaticConnection: function() {
@@ -884,25 +884,27 @@ NMDeviceBluetooth.prototype = {
         this._autoConnectionName = this._makeConnectionName(this.device);
 
         this._clearSection();
-        this._createSection();
+        this._queueCreateSection();
+        this._updateStatusItem();
+    },
+
+    _getDescription: function() {
+        return this.device.name || _("Bluetooth");
     }
-};
+});
 
 
 // Not a real device, but I save a lot code this way
-function NMDeviceVPN() {
-    this._init.apply(this, arguments);
-}
-
-NMDeviceVPN.prototype = {
-    __proto__: NMDevice.prototype,
+const NMDeviceVPN = new Lang.Class({
+    Name: 'NMDeviceVPN',
+    Extends: NMDevice,
 
     _init: function(client) {
         // Disable autoconnections
         this._autoConnectionName = null;
         this.category = NMConnectionCategory.VPN;
 
-        NMDevice.prototype._init.call(this, client, null, [ ]);
+        this.parent(client, null, [ ]);
     },
 
     connectionValid: function(connection) {
@@ -918,7 +920,7 @@ NMDeviceVPN.prototype = {
     },
 
     setActiveConnection: function(activeConnection) {
-        NMDevice.prototype.setActiveConnection.call(this, activeConnection);
+        this.parent(activeConnection);
 
         this.emit('active-connection-changed');
     },
@@ -935,14 +937,11 @@ NMDeviceVPN.prototype = {
     getStatusLabel: function() {
         return null;
     }
-};
+});
 
-function NMDeviceWireless() {
-    this._init.apply(this, arguments);
-}
-
-NMDeviceWireless.prototype = {
-    __proto__: NMDevice.prototype,
+const NMDeviceWireless = new Lang.Class({
+    Name: 'NMDeviceWireless',
+    Extends: NMDevice,
 
     _init: function(client, device, connections) {
         this.category = NMConnectionCategory.WIRELESS;
@@ -1014,7 +1013,7 @@ NMDeviceWireless.prototype = {
         this._apAddedId = device.connect('access-point-added', Lang.bind(this, this._accessPointAdded));
         this._apRemovedId = device.connect('access-point-removed', Lang.bind(this, this._accessPointRemoved));
 
-        NMDevice.prototype._init.call(this, client, device, validConnections);
+        this.parent(client, device, validConnections);
     },
 
     destroy: function() {
@@ -1034,7 +1033,7 @@ NMDeviceWireless.prototype = {
             this._apRemovedId = 0;
         }
 
-        NMDevice.prototype.destroy.call(this);
+        this.parent();
     },
 
     setEnabled: function(enabled) {
@@ -1235,7 +1234,6 @@ NMDeviceWireless.prototype = {
                       accessPoints: [ accessPoint ]
                     };
             apObj.ssidText = ssidToLabel(apObj.ssid);
-            needsupdate = true;
         }
 
         // check if this enables new connections for this group
@@ -1250,37 +1248,13 @@ NMDeviceWireless.prototype = {
             }
         }
 
-        if (needsupdate) {
-            if (apObj.item)
-                apObj.item.destroy();
-
+        if (pos == -1 || needsupdate) {
             if (pos != -1)
                 this._networks.splice(pos, 1);
+            pos = Util.insertSorted(this._networks, apObj, this._networkSortFunction);
 
-            if (this._networks.length == 0) {
-                // only network in the list
-                this._networks.push(apObj);
-                this._clearSection();
-                this._createSection();
-                return;
-            }
-
-            // skip networks that should appear earlier
-            let menuPos = 0;
-            for (pos = 0;
-                 pos < this._networks.length &&
-                 this._networkSortFunction(this._networks[pos], apObj) < 0; ++pos) {
-                if (this._networks[pos] != this._activeNetwork)
-                    menuPos++;
-            }
-
-            // (re-)add the network
-            this._networks.splice(pos, 0, apObj);
-
-            if (this._shouldShowConnectionList()) {
-                menuPos += (this._activeConnectionItem ? 1 : 0);
-                this._createNetworkItem(apObj, menuPos);
-            }
+            this._clearSection();
+            this._queueCreateSection();
         }
     },
 
@@ -1348,7 +1322,7 @@ NMDeviceWireless.prototype = {
     },
 
     _clearSection: function() {
-        NMDevice.prototype._clearSection.call(this);
+        this.parent();
 
         for (let i = 0; i < this._networks.length; i++)
             this._networks[i].item = null;
@@ -1367,7 +1341,7 @@ NMDeviceWireless.prototype = {
         let obj = this._connections[pos];
         this._connections.splice(pos, 1);
 
-        let anyauto = false, forceupdate = false;
+        let forceupdate = false;
         for (let i = 0; i < this._networks.length; i++) {
             let apObj = this._networks[i];
             let connections = apObj.connections;
@@ -1375,16 +1349,14 @@ NMDeviceWireless.prototype = {
                 if (connections[k]._uuid == connection._uuid) {
                     // remove the connection from the access point group
                     connections.splice(k);
-                    anyauto = connections.length == 0;
+                    forceupdate = forceupdate || connections.length == 0;
 
-                    if (anyauto) {
-                        // this potentially changes the sorting order
-                        forceupdate = true;
+                    if (forceupdate)
                         break;
-                    }
+
                     if (apObj.item) {
                         if (apObj.item instanceof PopupMenu.PopupSubMenuMenuItem) {
-                            let items = apObj.item.menu.getMenuItems();
+                            let items = apObj.item.menu._getMenuItems();
                             if (items.length == 2) {
                                 // we need to update the connection list to convert this to a normal item
                                 forceupdate = true;
@@ -1406,10 +1378,10 @@ NMDeviceWireless.prototype = {
             }
         }
 
-        if (forceupdate || anyauto) {
+        if (forceupdate) {
             this._networks.sort(this._networkSortFunction);
             this._clearSection();
-            this._createSection();
+            this._queueCreateSection();
         }
     },
 
@@ -1442,13 +1414,13 @@ NMDeviceWireless.prototype = {
         if (forceupdate) {
             this._networks.sort(this._networkSortFunction);
             this._clearSection();
-            this._createSection();
+            this._queueCreateSection();
         }
     },
 
     _createActiveConnectionItem: function() {
         let icon, title;
-        if (this._activeConnection._connection) {
+        if (this._activeConnection && this._activeConnection._connection) {
             let connection = this._activeConnection._connection;
             if (this._activeNetwork)
                 this._activeConnectionItem = new NMNetworkMenuItem(this._activeNetwork.accessPoints, undefined,
@@ -1541,7 +1513,7 @@ NMDeviceWireless.prototype = {
         if (!this._shouldShowConnectionList())
             return;
 
-        if(this._activeConnection) {
+        if (this._activeNetwork) {
             this._createActiveConnectionItem();
             this.section.addMenuItem(this._activeConnectionItem);
         }
@@ -1550,22 +1522,22 @@ NMDeviceWireless.prototype = {
 
         for(let j = 0; j < this._networks.length; j++) {
             let apObj = this._networks[j];
-            if (apObj == this._activeNetwork)
+            if (apObj == this._activeNetwork) {
+                activeOffset--;
                 continue;
+            }
 
             this._createNetworkItem(apObj, j + activeOffset);
         }
     },
-};
+});
 
-function NMApplet() {
-    this._init.apply(this, arguments);
-}
-NMApplet.prototype = {
-    __proto__: PanelMenu.SystemStatusButton.prototype,
+const NMApplet = new Lang.Class({
+    Name: 'NMApplet',
+    Extends: PanelMenu.SystemStatusButton,
 
     _init: function() {
-        PanelMenu.SystemStatusButton.prototype._init.call(this, 'network-error');
+        this.parent('network-error', null);
 
         this._client = NMClient.Client.new();
 
@@ -2065,10 +2037,11 @@ NMApplet.prototype = {
                         }
                         this.setIcon('network-wireless-connected');
                     } else {
-                        if (this._accessPointUpdateId && this._activeAccessPoint != ap) {
-                            this._activeAccessPoint.disconnect(this._accessPointUpdateId);
+                        if (this._activeAccessPoint != ap) {
+                            if (this._accessPointUpdateId)
+                                this._activeAccessPoint.disconnect(this._accessPointUpdateId);
                             this._activeAccessPoint = ap;
-                            this._activeAccessPointUpdateId = ap.connect('notify::strength', Lang.bind(function() {
+                            this._activeAccessPointUpdateId = ap.connect('notify::strength', Lang.bind(this, function() {
                                 this.setIcon('network-wireless-signal-' + signalToIcon(ap.strength));
                             }));
                         }
@@ -2095,8 +2068,9 @@ NMApplet.prototype = {
                     break;
                 }
 
-                if (this._mobileUpdateId && this._mobileUpdateDevice != dev) {
-                    this._mobileUpdateDevice.disconnect(this._mobileUpdateId);
+                if (dev.mobileDevice != this._mobileUpdateDevice) {
+                    if (this._mobileUpdateId)
+                        this._mobileUpdateDevice.disconnect(this._mobileUpdateId);
                     this._mobileUpdateDevice = dev.mobileDevice;
                     this._mobileUpdateId = dev.mobileDevice.connect('notify::signal-quality', Lang.bind(this, function() {
                         this.setIcon('network-cellular-signal-' + signalToIcon(dev.mobileDevice.signal_quality));
@@ -2129,17 +2103,14 @@ NMApplet.prototype = {
             this._mobileUpdateId = 0;
         }
     }
-};
+});
 
-function NMMessageTraySource() {
-    this._init();
-}
-
-NMMessageTraySource.prototype = {
-    __proto__: MessageTray.Source.prototype,
+const NMMessageTraySource = new Lang.Class({
+    Name: 'NMMessageTraySource',
+    Extends: MessageTray.Source,
 
     _init: function() {
-        MessageTray.Source.prototype._init.call(this, _("Network Manager"));
+        this.parent(_("Network Manager"));
 
         let icon = new St.Icon({ icon_name: 'network-transmit-receive',
                                  icon_type: St.IconType.SYMBOLIC,
@@ -2147,4 +2118,4 @@ NMMessageTraySource.prototype = {
                                });
         this._setSummaryIcon(icon);
     }
-};
+});
